version 9
program stcoxsim
	syntax varlist	, [hyp(real 0.0) , strata(string) , reps(integer 6000) , noBS(integer 0) ]

	preserve 	
	pause on
	global V_all "`varlist'"	// copy to global macros
	global hypo "`hyp'"
	global nreps "`reps'"
	global noBS = "`noBS'" 
	if "`strata'" == "" {
		// default to one stratum if str() is not specified
		capture drop constant_of_ones
		gen constant_of_ones = 1
		global str_var  "constant_of_ones"
		}
		else global str_var "`strata'"

	// following puts first var of vlist into V_interest and 
	// the remaining vars into V_np
	gettoken  (global) V_interest  (global) V_np :  varlist 

	set more off
	scalar drop _all
	mat drop _all
	mata : mata clear
	run mata_defs
	scalar ntrial = 0
	scalar nskov = 2000

	// set seed from clock
	scalar clks=c(current_time)
	scalar sd = 10^4*real(substr(clks,1,2))+10^2*real(substr(clks,4,2))+real(substr(clks,7,2))
	local newseed = sd
	set seed `newseed'

	scalar noPvalsim = $noBS
	// scalar noPvalsim = 0    // set to 1 to omit this part
	scalar noSkov = 0

	// this sorting is only to allow for conditioning on cens config
	// by using original failure indicator _d_orig from this time-sorted data
	// in the new time-sorted data for all simulation trials

	sort $str_var _t			// sort data in order of failure times
								// within strata	  
	gen _d_orig = _d 
	mkmat _d_orig		// put in a matrix that will not be sorted later 
	drop _d_orig
	
  quietly: stcox $V_all , efron nohr  strata($str_var)			// fit full model
												// now using Efron for ties
	scalar nval = e(N)
	matrix vari_hat_data = e(V)
	scalar llik_full_data = e(ll)
	matrix beta_hat_data = e(b)
	capture drop logscale
	matrix score logscale = beta_hat_data	// linear predictor
	capture drop scale_hat_data 
	gen scale_hat_data = exp(-logscale) 		// scaling to use in Skov things
	matrix beta_hat_data = beta_hat_data'
	scalar psihat = beta_hat_data[1,1]

	scalar psihyp = $hypo
	capture drop ofstvar
	gen ofstvar = $V_interest * psihyp 

 	quietly: stcox $V_np , efron nohr offset(ofstvar)	 strata($str_var)  // fit hyp model 
	matrix vari_til_data = e(V)
	scalar llik_hyp_data = e(ll)
	matrix beta_til_data = e(b)
	capture drop logscale
	mat score logscale = beta_til_data
	capture drop scale_til_data
	gen scale_til_data = exp(-(logscale + ofstvar))		// scaling to use in rvals sim
	mat beta_til_data = beta_til_data' 
	// put the hyp element in beta_til
	mat tmp = beta_hat_data
	mat tmp[1,1] = psihyp
	mat tmp[2,1] = beta_til_data
	mat beta_til_data = tmp

	// get rval for observed data
	scalar sign = (-1) ^ ( psihat < psihyp )
	scalar rval_data = sign * sqrt( 2 * (llik_full_data - llik_hyp_data) ) 


	scalar Wald_data = (psihat - psihyp) / sqrt(vari_hat_data[1,1])
	scalar INF_frs = log(Wald_data/rval_data) / rval_data 
	mat info_hat_data = inv(vari_hat_data)
	mat NP_info_hat_data = info_hat_data[2...,2...]
	mat info_til_data = inv(vari_til_data)
	mat rho_num = det(NP_info_hat_data) 
	scalar rho_num = rho_num[1,1]
	mat	rho_den = det(info_til_data)
	scalar rho_den = rho_den[1,1]
	scalar rho = rho_num / rho_den
	scalar NP_frs = 0.5 * log(rho) / rval_data
	scalar rstar_FRS = rval_data + NP_frs + INF_frs

   
	capture : program drop simstep
	run simdef			// load program simstep

	// initialize matrices to accumulate things for Skov covars
	mata: init_accum()

	capture: erase rvals_sim.dta
	if (noPvalsim==1) {
		simulate rvals = r(rval) , reps(2000)  : simstep
		}
	else {
		simulate rvals = r(rval) , reps($nreps)  saving(rvals_sim) : simstep
		}

	display "init seed was  " sd
	if (noSkov==0) 	mata: rstar_calc()
	mata: Pval_calc()
	if (noPvalsim==0) {
		quietly : histogram rvals
		}
	pause off
end

															




